VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "ser_sr24"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'   ser_sr24.cls
'   This file is the serial board class module for use in Visual Basic
'   It contains all the properties and methods which provide access to the
'   serial board in Visual Basic.
'
'   Revised:  November 2000
'   By:  BMB
'
'   Winford Engineering
'   www.winfordeng.com


'************************************************
' Global class variables
'************************************************
Private open_flag As Long       'indicates if serial board is open or closed
Private handle As Long          'handle to serial board
Private time_out As Long        'timeout value for the board (in milliseconds)

'************************************************
' Class events
'************************************************
Public Event HardwareEvent(ByVal id As Long, ByVal value As Long)

Public Property Let Timeout(ByVal value As Long)
    'Sets the timeout property for the board object
    '--------------------------------------------
    'Parameters: value - new timeout value
    'Returns:
    '--------------------------------------------
    'Variable Declarations
    'Global
        'handle
        'time_out
    'Local
        Dim result As Long       'result of function call
        
    'Check if board is open and exit if it is not
    If open_flag = 0 Then
        Exit Property
    End If
    
    If (value > 0) Then
        'Valid timeout value for Visual Basic
        result = ser_sr24_set_timeout(handle, value)
        If result = 0 Then
            'Successfully set the timeout value --> update object property
            time_out = value
        End If
    End If
    
End Property
Public Property Get Timeout() As Long
    'Returns the timeout property for the board object
    '--------------------------------------------
    'Parameters:
    'Returns:
    '--------------------------------------------
    'Variable Declarations
    'Global
        'time_out
    'Local
    
    Timeout = time_out
End Property
Public Function OpenBoard(ByVal serial As String, ByVal baud As BaudRate) As Long
    'Attempts to open a connection to the serial board and stores the handle
    'if successful
    'Also registers our VB callback for use in hardware events
    '--------------------------------------------
    'Parameters: serial - indicates the serial port to use ('COM1', 'COM2', etc)
    '            baud - indicates the desired baud rate (predefined constant, not actual baud value)
    'Returns: 0 for success, error code for error
    '--------------------------------------------
    'Variable Declarations
    'Global
        'handle
        'open_flag
    'Local
        Dim result As Long                  'result of function call
        Dim event_spec As ser_event_spec    'defines hardware event handler (callback, etc)
        
    'Open the board
    handle = ser_sr24_open(serial, baud, result)
    
    If result = SER_SUCCESS Then
        'Successfully opened the card
        open_flag = 1
        'Register callback for hardware events
            'First, grab address of serial board object; this is needed
            'so that we can call an object event from the callback
            event_spec.extra = ser_returnvalue(Me)
            'Use callback, utilizing Windows messages
            event_spec.type = 3
            'Provide address of callback function; we need to store the address
            'in a long, but 'AddressOf' returns type 'any'; we use a function to
            'translate type 'any' to type 'long'
            event_spec.eventfn = ser_any2long(AddressOf hw_callback)
            'Finally, pass the structure to the API
            ser_sr24_set_event_handler handle, event_spec
     Else
        'Error opening board
        open_flag = 0
    End If
        
    OpenBoard = result
    Exit Function
        
End Function

Public Function VerifyConnection() As Long
    'Verifies that the serial connection to the board is good
    '--------------------------------------------
    'Parameters: none
    'Returns: 0 for success, error code for error
    '--------------------------------------------
    'Variable Declarations
    'Global
        'handle
        'open_flag
    'Local
        Dim result As Long       'result of function call

    'Check if board is open and return error if it is not
    If open_flag = 0 Then
        VerifyConnection = SER_INVALID_HANDLE
        Exit Function
    End If

    result = ser_sr24_verify_connection(handle)
    
    VerifyConnection = result
    Exit Function
    
End Function

Public Function ResetBoard() As Long
    'Sends a reset command to the serial board
    '--------------------------------------------
    'Parameters: none
    'Returns: 0 for success, error code for error
    '--------------------------------------------
    'Variable Declarations
    'Global
        'handle
        'open_flag
    'Local
        Dim result As Long       'result of function call

    'Check if board is open and return error if it is not
    If open_flag = 0 Then
        ResetBoard = SER_INVALID_HANDLE
        Exit Function
    End If

    'Reset the board
    result = ser_sr24_reset_board(handle)
    
    ResetBoard = result
    Exit Function
    
End Function

Public Function OutputByte(ByVal port As Long, ByVal value As Long) As Long
    'Outputs a value to the specified port
    '--------------------------------------------
    'Parameters: port - which port to write to
    '            value - value to write to the port
    'Returns: 0 for success, error code for error
    '--------------------------------------------
    'Variable Declarations
    'Global
        'handle
        'open_flag
    'Local
        Dim result As Long       'result of function call

    'Check if board is open and return error if it is not
    If open_flag = 0 Then
        OutputByte = SER_INVALID_HANDLE
        Exit Function
    End If
    
    result = ser_sr24_output_byte(handle, port, value)
    OutputByte = result
    
    Exit Function
    
End Function

Public Function InputByte(ByVal port As Long, ByRef value As Long) As Long
    'Reads a value from the specified port
    '--------------------------------------------
    'Parameters: port - which port to write to
    '            value - variable where the read value is stored
    'Returns: 0 for success, error code for error
    '--------------------------------------------
    'Variable Declarations
    'Global
        'handle
        'open_flag
    'Local
        Dim result As Long        'result of function call

    'Check if board is open and return error if it is not
    If open_flag = 0 Then
        InputByte = SER_INVALID_HANDLE
        Exit Function
    End If
    
    result = ser_sr24_input_byte(handle, port, value)
    InputByte = result
    
    Exit Function
    
End Function

Public Function OutputBit(ByVal port As Long, ByVal bit As Long, ByVal value As Long) As Long
    'Outputs a value to the specified bit
    '--------------------------------------------
    'Parameters: port - which port to write to
    '            bit - specific bit to write to
    '            value - value to write to the bit
    'Returns: 0 for success, error code for error
    '--------------------------------------------
    'Variable Declarations
    'Global
        'handle
        'open_flag
    'Local
        Dim result As Long       'result of function call

    'Check if board is open and return error if it is not
    If open_flag = 0 Then
        OutputBit = SER_INVALID_HANDLE
        Exit Function
    End If
    
    result = ser_sr24_output_bit(handle, port, bit, value)
    OutputBit = result
    Exit Function

End Function

Public Function InputBit(ByVal port As Long, ByVal bit As Long, ByRef value As Long) As Long
    'Reads a value from the specified bit
    '--------------------------------------------
    'Parameters: port - which port to read
    '            bit - which bit to read
    '            value - variable where the read value is stored
    'Returns: 0 for success, error code for error
    '--------------------------------------------
    'Variable Declarations
    'Global
        'handle
        'open_flag
    'Local
        Dim result As Long       'result of function call

    'Check if board is open and return error if it is not
    If open_flag = 0 Then
        InputBit = SER_INVALID_HANDLE
        Exit Function
    End If
    
    result = ser_sr24_input_bit(handle, port, bit, value)
    InputBit = result
    Exit Function

End Function

Public Function InputAnalog(ByVal channel As Long, ByRef value As Long) As Long
    'Reads a channel of the A/D converter (Port 1 only)
    '--------------------------------------------
    'Parameters: channel - which analog channel to read
    '            value - variable where the read value is stored
    'Returns: 0 for success, error code for error
    '--------------------------------------------
    'Variable Declarations
    'Global
        'handle
        'open_flag
    'Local
        Dim result  As Long      'result of function call

    'Check if board is open and return error if it is not
    If open_flag = 0 Then
        InputAnalog = SER_INVALID_HANDLE
        Exit Function
    End If
    
    result = ser_sr24_input_analog(handle, channel, value)
    InputAnalog = result
    Exit Function

End Function

Public Function GetPortMode(ByVal port As Long, ByVal property As PortProperty) As Long
    'Reads the current configuration of a particular property of a port on the serial board
    '--------------------------------------------
    'Parameters: port - which port on the serial board to report on
    '            property - indicates which property of the port we want to report on
    'Returns: value for the property, or error code for error
    '--------------------------------------------
    'Variable Declarations
    'Global
        'handle
        'open_flag
    'Local
        Dim result As Long       'result of function call
        
    'Check if board is open and return error if it is not
    If open_flag = 0 Then
        GetPortMode = SER_INVALID_HANDLE
        Exit Function
    End If
    
    result = ser_sr24_get_portmode(handle, port, property)
    
    GetPortMode = result
    Exit Function

End Function

Public Function SetPortMode(ByVal port As Long, ByVal property As PortProperty, ByVal mode As PortMode) As Long
    'Configures a particular property of a port on the serial board
    '--------------------------------------------
    'Parameters: port - which port on the serial board to configure
    '            property - indicates which property of the port we want to configure
    '            mode - value to assign to the property
    'Returns: 0 for success, error code for error
    '--------------------------------------------
    'Variable Declarations
    'Global
        'handle
        'open_flag
    'Local
        Dim result As Long       'result of function call
        
    'Check if board is open and return error if it is not
    If open_flag = 0 Then
        SetPortMode = SER_INVALID_HANDLE
        Exit Function
    End If
    
    result = ser_sr24_set_portmode(handle, port, property, mode)
    
    SetPortMode = result
    Exit Function
    
End Function

Public Function WaitEvent(ByRef events() As Long, ByVal Timeout As Long, ByRef value As Long) As Long
    'Waits for any of the specified events to occur and then returns (or times out and returns)
    '--------------------------------------------
    'Parameters: events - array of event ids to wait for
    '            timeout - time (in milliseconds) before function will timeout (0 for no timeout)
    '            value - variable where the event value is stored
    'Returns: id of event which occurred, or error code for error
    '--------------------------------------------
    'Variable Declarations
    'Global
        'handle
        'open_flag
    'Local
        Dim result As Long      'result of function call
        Dim size As Long        'array size
        
    'Check if board is open and return error if it is not
    If open_flag = 0 Then
        WaitEvent = SER_INVALID_HANDLE
        Exit Function
    End If
    
    'Calculate size of array
    size = UBound(events) - LBound(events) + 1
    
    result = ser_sr24_wait_event(handle, events(1), size, Timeout, value)
    WaitEvent = result
    Exit Function
    
End Function

Public Function CloseBoard() As Long
    'Closes the connection to the serial board
    '--------------------------------------------
    'Parameters: none
    'Returns: 0
    '--------------------------------------------
    'Variable Declarations
    'Global
        'handle
        'open_flag
    'Local
        Dim result As Long          ' result of function call
        
    'Close the board if it is open
    If open_flag = 1 Then
        result = ser_sr24_close(handle)
        open_flag = 0
    End If
    
    CloseBoard = 0
    Exit Function

End Function

Public Function ErrorString(ByVal errorcode As Long) As String
    'Translates the passed error code into a meaningful error message
    '--------------------------------------------
    'Parameters: error code
    'Returns: error string
    '--------------------------------------------
    'Variable Declarations
    'Global
    'Local
        Dim str_ptr As Long         'pointer to error string
        Dim err_str_len             'length of error string
        Dim str As String           'holds error string
        
    'Have to do some weird stuff to translate between C and VB strings
    
    'Get pointer to error string
    str_ptr = ser_sr24_error_string(errorcode)
    
    'Get string length
    err_str_len = mod_ser_lstrlen(str_ptr)
    
    'Create buffer to hold error string
    str = Space(err_str_len + 1)
        
    'Copy error string into buffer
    mod_ser_lstrcpy str, str_ptr
    
    ErrorString = str
    Exit Function
        
End Function

Public Function RegisterEvent(ByVal port As Long, ByVal bit As Long, ByVal id As Long) As Long
    'Registers an event with the serial board
    '--------------------------------------------
    'Parameters: port - which port on the serial board to set up an event for
    '            bit - which bit of the port to set up an event for
    '            id - user-assigned id for this event; passed to event handler when event occurs
    '                 so we know exactly which event occurred
    'Returns: 0 for success, error code for error
    '--------------------------------------------
    'Variable Declarations
    'Global
        'handle
        'open_flag
    'Local
        Dim result As Long       'result of function call
        
    'Check if board is open and return error if it is not
    If open_flag = 0 Then
        RegisterEvent = SER_INVALID_HANDLE
        Exit Function
    End If

    result = ser_sr24_register_event(handle, port, bit, id)
    RegisterEvent = result
    Exit Function

End Function

Public Function TriggerEvent(ByVal id As Long, value As Long)
    'Raises an event; when a hardware event occurs, a VB callback function
    'executes, which calls this object method, which raises the event.
    'We do it this way because we can raise an event only from within the class code.
    'Note that this provides an easy way to simulate hardware events; just call this method
    '--------------------------------------------
    'Parameters: id - user-assigned id for this event
    '            value - value associated with the event (value on the port, etc)
    'Returns: 0
    '--------------------------------------------
    'Variable Declarations
    'Global
    'Local
        
    RaiseEvent HardwareEvent(id, value)
    
    TriggerEvent = 0
    Exit Function
    
End Function

Private Sub Class_Terminate()
    'Automatically executes before the object is destroyed.
    'We check to be sure that the ser_sr24_close() function has been called before
    'destroying the object. This is necessary to ensure that event callbacks do not
    'continue to occur, causing a crash.
    '--------------------------------------------
    'Parameters: none
    '--------------------------------------------
    'Variable Declarations
    'Global
        'handle
        'open_flag
    'Local
        
    'Check if board is open
    If open_flag = 1 Then
        'Board is still open --> close it
        Me.CloseBoard
    End If
End Sub
